@@ -2,7 +2,7 @@
 
   VSEARCH: a versatile open source tool for metagenomics
 
-  Copyright (C) 2014-2024, Torbjorn Rognes, Frederic Mahe and Tomas Flouri
+  Copyright (C) 2014-2025, Torbjorn Rognes, Frederic Mahe and Tomas Flouri
   All rights reserved.
 
   Contact: Torbjorn Rognes <torognes@ifi.uio.no>,
@@ -58,5 +58,5 @@
 
 */
 
-auto fastq_eestats() -> void;
-auto fastq_eestats2() -> void;
+auto fastq_eestats(struct Parameters const & parameters) -> void;
+auto fastq_eestats2(struct Parameters const & parameters) -> void;
