@@ -2,7 +2,7 @@
 
   VSEARCH: a versatile open source tool for metagenomics
 
-  Copyright (C) 2014-2024, Torbjorn Rognes, Frederic Mahe and Tomas Flouri
+  Copyright (C) 2014-2025, Torbjorn Rognes, Frederic Mahe and Tomas Flouri
   All rights reserved.
 
   Contact: Torbjorn Rognes <torognes@ifi.uio.no>,
@@ -76,12 +76,12 @@ struct minheap_s
 
 using minheap_t = struct minheap_s;
 
-inline auto minheap_isempty(minheap_t * a_minheap) -> bool
+inline auto minheap_isempty(minheap_t const * a_minheap) -> bool
 {
   return (a_minheap->count == 0);
 }
 
-inline auto minheap_empty(minheap_t * a_minheap) -> void
+inline auto minheap_clear(minheap_t * a_minheap) -> void
 {
   a_minheap->count = 0;
 }
