@@ -2,7 +2,7 @@
 
   VSEARCH: a versatile open source tool for metagenomics
 
-  Copyright (C) 2014-2024, Torbjorn Rognes, Frederic Mahe and Tomas Flouri
+  Copyright (C) 2014-2025, Torbjorn Rognes, Frederic Mahe and Tomas Flouri
   All rights reserved.
 
   Contact: Torbjorn Rognes <torognes@ifi.uio.no>,
@@ -63,58 +63,60 @@
 
 
 auto results_show_alnout(std::FILE * output_handle,
-                         struct hit * hits,
+                         struct hit const * hits,
                          int hitcount,
-                         char * query_head,
-                         char * qsequence,
+                         char const * query_head,
+                         char const * qsequence,
                          int64_t qseqlen) -> void;
 
 auto results_show_lcaout(std::FILE * output_handle,
-                         struct hit * hits,
+                         struct hit const * hits,
                          int hitcount,
-                         char * query_head) -> void;
+                         char const * query_head) -> void;
 
 auto results_show_blast6out_one(std::FILE * output_handle,
-                                struct hit * hits,
-                                char * query_head,
+                                struct hit const * hits,
+                                char const * query_head,
                                 int64_t qseqlen) -> void;
 
 auto results_show_uc_one(std::FILE * output_handle,
-                         struct hit * hits,
-                         char * query_head,
+                         struct hit const * hits,
+                         char const * query_head,
                          int64_t qseqlen,
                          int clusterno) -> void;
 
 auto results_show_userout_one(std::FILE * output_handle,
-                              struct hit * hits,
-                              char * query_head,
-                              char * qsequence,
+                              struct hit const * hits,
+                              char const * query_head,
+                              char const * qsequence,
                               int64_t qseqlen,
-                              char * qsequence_rc) -> void;
+                              char const * qsequence_rc) -> void;
 
 auto results_show_fastapairs_one(std::FILE * output_handle,
-                                 struct hit * hits,
-                                 char * query_head,
-                                 char * qsequence,
-                                 char * qsequence_rc) -> void;
+                                 struct hit const * hits,
+                                 char const * query_head,
+                                 char const * qsequence,
+                                 char const * qsequence_rc) -> void;
 
 auto results_show_qsegout_one(std::FILE * output_handle,
-                              struct hit * hits,
-                              char * query_head,
-                              char * qsequence,
+                              struct hit const * hits,
+                              char const * query_head,
+                              char const * qsequence,
                               int64_t qseqlen,
-                              char * qsequence_rc) -> void;
+                              char const * qsequence_rc) -> void;
 
 auto results_show_tsegout_one(std::FILE * output_handle,
-                              struct hit * hits) -> void;
+                              struct hit const * hits) -> void;
 
 auto results_show_samheader(std::FILE * output_handle,
-                            char * cmdline,
-                            char * dbname) -> void;
+                            char const * cmdline,
+                            char const * dbname) -> void;
 
 auto results_show_samout(std::FILE * output_handle,
-                         struct hit * hits,
+                         struct hit const * hits,
                          int hitcount,
-                         char * query_head,
-                         char * qsequence,
-                         char * qsequence_rc) -> void;
+                         char const * query_head,
+                         char const * qsequence,
+                         char const * qsequence_rc) -> void;
+
+auto clean_up() -> void;
