@@ -2,7 +2,7 @@
 
   VSEARCH: a versatile open source tool for metagenomics
 
-  Copyright (C) 2014-2024, Torbjorn Rognes, Frederic Mahe and Tomas Flouri
+  Copyright (C) 2014-2025, Torbjorn Rognes, Frederic Mahe and Tomas Flouri
   All rights reserved.
 
   Contact: Torbjorn Rognes <torognes@ifi.uio.no>,
@@ -58,9 +58,11 @@
 
 */
 
+#include "linmemalign.h"
 #include <array>
 #include <vector>
 
+
 struct uhandle_s;
 
 /* the number of alignments that can be delayed */
@@ -133,9 +135,11 @@ struct searchinfo_s
   char * query_head = nullptr;            /* query header */
   int qseqlen = 0;                  /* query length */
   int seq_alloc = 0;                /* bytes allocated for the query sequence */
+  std::vector<char> qsequence_v;  /* vector of query sequence chars */
   char * qsequence = nullptr;             /* query sequence */
   unsigned int kmersamplecount = 0; /* number of kmer samples from query */
-  unsigned int * kmersample = nullptr;    /* list of kmers sampled from query */
+  unsigned int const * kmersample = nullptr;    /* list of kmers sampled from query */
+  std::vector<count_t> kmers_v; /* vector of kmer counts */
   count_t * kmers = nullptr;              /* list of kmer counts for each db seq */
   std::vector<struct hit> hits_v; /* vector of hits */
   struct hit * hits = nullptr;            /* list of hits */
@@ -150,9 +154,9 @@ struct searchinfo_s
   int finalized = 0;
 };
 
-auto search_topscores(struct searchinfo_s * si) -> void;
+auto search_topscores(struct searchinfo_s * searchinfo) -> void;
 
-auto search_onequery(struct searchinfo_s * si, int seqmask) -> void;
+auto search_onequery(struct searchinfo_s * searchinfo, int seqmask) -> void;
 
 auto search_findbest2_byid(struct searchinfo_s * si_p,
                            struct searchinfo_s * si_m) -> struct hit *;
@@ -160,18 +164,17 @@ auto search_findbest2_byid(struct searchinfo_s * si_p,
 auto search_findbest2_bysize(struct searchinfo_s * si_p,
                              struct searchinfo_s * si_m) -> struct hit *;
 
-auto search_acceptable_unaligned(struct searchinfo_s * si,
+auto search_acceptable_unaligned(struct searchinfo_s const & searchinfo,
                                  int target) -> bool;
 
-auto search_acceptable_aligned(struct searchinfo_s * si,
+auto search_acceptable_aligned(struct searchinfo_s const & searchinfo,
                                struct hit * hit) -> bool;
 
 auto align_trim(struct hit * hit) -> void;
 
 auto search_joinhits(struct searchinfo_s * si_p,
                      struct searchinfo_s * si_m,
-                     struct hit * * hits,
-                     int * hit_count) -> void;
+                     std::vector<struct hit> & hits) -> void;
 
-auto search_enough_kmers(struct searchinfo_s * si,
+auto search_enough_kmers(struct searchinfo_s const & searchinfo,
                          unsigned int count) -> bool;
