@@ -2,7 +2,7 @@
 
   VSEARCH: a versatile open source tool for metagenomics
 
-  Copyright (C) 2014-2024, Torbjorn Rognes, Frederic Mahe and Tomas Flouri
+  Copyright (C) 2014-2025, Torbjorn Rognes, Frederic Mahe and Tomas Flouri
   All rights reserved.
 
   Contact: Torbjorn Rognes <torognes@ifi.uio.no>,
@@ -59,6 +59,7 @@
 */
 
 #include "vsearch.h"
+#include "utils/fatal.hpp"
 #include <algorithm>  // std::count_if
 #include <cassert>
 #include <cinttypes>  // macros PRIu64 and PRId64
@@ -114,23 +115,28 @@ struct file_types {
 
 //   std::random_device r;
 //   std::default_random_engine generator(r());
-//   std::vector<int> v1 = {1000, 100, 10, 1};
-//   std::discrete_distribution<int> distribution(v1.begin(), v1.end());
+//   std::vector<int> weights = {1000, 100, 10, 1};
+//   std::discrete_distribution<unsigned long long> distribution(weights.begin(), weights.end());
 
-//   std::vector<int> v2(v1.size());
+//   std::vector<int> v2(weights.size());
 
-//   for (auto i=0ULL; i<nreads; ++i) {
-//     int const number = distribution(generator);
+//   for (auto i = 0ULL; i < nreads; ++i) {
+//     auto const number = distribution(generator);
 //     ++v2[number];
 //   }
 
-//   for (auto i=0UL; i<v2.size(); ++i)
+//   for (auto i = 0UL; i < v2.size(); ++i)
 //     std::cout << i << ": " << v1[i] << " " << v2[i] << "\n";
 
 //   return 0;
 // }
 
 
+// refactoring: "RTK: efficient rarefaction analysis of large datasets"
+//  a fast C++11 rarefaction tool https://github.com/hildebra/Rarefaction
+//  maybe they use a better approach than the std::discrete_distribution?
+
+
 auto open_output_files(struct file_types & ouput_files) -> void {
   if (ouput_files.fasta.kept.name != nullptr) {
     ouput_files.fasta.kept.handle = fopen_output(ouput_files.fasta.kept.name);
@@ -358,7 +364,7 @@ auto close_output_files(struct file_types const & ouput_files) -> void {
 
 
 auto subsample(struct Parameters const & parameters) -> void {
-  struct file_types ouput_files;
+  struct file_types ouput_files;  // refactoring: direct initialization {}; then struct can be const
   ouput_files.fasta.kept.name = parameters.opt_fastaout;
   ouput_files.fasta.lost.name = parameters.opt_fastaout_discarded;
   ouput_files.fastq.kept.name = parameters.opt_fastqout;
