@@ -2,7 +2,7 @@
 
   VSEARCH: a versatile open source tool for metagenomics
 
-  Copyright (C) 2014-2024, Torbjorn Rognes, Frederic Mahe and Tomas Flouri
+  Copyright (C) 2014-2025, Torbjorn Rognes, Frederic Mahe and Tomas Flouri
   All rights reserved.
 
   Contact: Torbjorn Rognes <torognes@ifi.uio.no>,
@@ -68,12 +68,12 @@ auto unique_exit(struct uhandle_s * unique_handle) -> void;
 auto unique_count(struct uhandle_s * unique_handle,
                   int wordlength,
                   int seqlen,
-                  char * seq,
+                  char const * seq,
                   unsigned int * listlen,
-                  unsigned int * * list,
+                  unsigned int const * * list,
                   int seqmask) -> void;
 
-auto unique_count_shared(struct uhandle_s * unique_handle,
+auto unique_count_shared(struct uhandle_s const & unique_handle,
                          int wordlength,
                          int listlen,
-                         unsigned int * list) -> int;
+                         unsigned int const * list) -> unsigned int;
